local L = LibStub('AceLocale-3.0'):GetLocale('TimeToDie', true)

local TimeToDie = LibStub('AceAddon-3.0'):NewAddon('TimeToDie', 'AceEvent-3.0', 'AceConsole-3.0')
_G.TimeToDie = TimeToDie

------------------------------
--      Initialization      --
------------------------------

local defaultFont = GameFontNormal:GetFont()
local events = {target = 'PLAYER_TARGET_CHANGED', focus = 'PLAYER_FOCUS_CHANGED'}
local healing = '|cff00ff00'..SHOW_COMBAT_HEALING ..'|r'

local UnitIsFriend = UnitIsFriend
local UnitHealth = UnitHealth
local GetTime = GetTime
local UnitHealtMax = UnitHealtMax
local format = format

local frame = TimeToDieFrame
local bg, text = frame.bg, frame.text


local defaults = {
   profile = {
      locked = false,
      enemy = true,
      target = 'target',
      font = defaultFont,
      size = 24,
      r = 1,
      g = .82,
      b = 0,
      outline = '',
      strata = 'LOW',
      update = 1,
   }
}

function TimeToDie:OnInitialize()
   local db = LibStub('AceDB-3.0'):New('TimeToDieDB', defaults, 'Default')
   self.db = db
   local RegisterCallback = db.RegisterCallback
   RegisterCallback(self, 'OnProfileChanged', 'ApplySettings')
   RegisterCallback(self, 'OnProfileCopied', 'ApplySettings')
   RegisterCallback(self, 'OnProfileReset', 'ApplySettings')

   self:RegisterOptions()
end

function TimeToDie:OnEnable()
	self:RegisterEvent('PLAYER_TARGET_CHANGED', 'OnChange')
	self:RegisterEvent('PLAYER_FOCUS_CHANGED', 'OnChange')
	self:RegisterEvent('UNIT_HEALTH')

	self:ApplySettings()

	frame:Show()
end

function TimeToDie:OnDisable()
   frame:Hide()
end

-------------------------------
--      Event Functions      --
-------------------------------

local health0, time0 -- initial health and time point
local mhealth, mtime -- current midpoint

function TimeToDie:UNIT_HEALTH(event, unit)
   local profile = self.db.profile
   local target = profile.target
	if unit ~= target then return end
   if profile.enemy and UnitIsFriend('player', target) then return end

	local health, time = UnitHealth(target), GetTime()

   if health == UnitHealthMax(target) then
      text:SetText(not profile.locked and 'TimeToDie' or nil)
      self.dataobj.text = 'TimeToDie'
      health0, time0, mhealth, mtime = nil
	end

	if not health0 then
		health0, time0 = health, time
		return
	end
	if mhealth then
		mhealth, mtime = (mhealth + health) / 2, (mtime + time) / 2
	else
		mhealth, mtime = health, time
	end

	local dhealth = mhealth - health0

   local res
   if dhealth ~= 0 then
		time = health * (time0 - mtime) / dhealth
      if time < 0 then
         res = healing
         health0, time0, mhealth, mtime = nil
		elseif time > 60 then
			res = format('%dm %ds', time/60, time%60)
		else
			res = format('%ds', time%60)
		end
	else
		res = '0s'
	end
   self.dataobj.text = text or 'TimeToDie'
   text:SetText(res)
end

function TimeToDie:OnChange(event)
   local profile = self.db.profile
	if events[profile.target] ~= event then return end
	text:SetText(not profile.locked and 'TimeToDie' or nil)
	self.dataobj.text = 'TimeToDie'
	health0, time0, mhealth, mtime = nil
end

-- don't need to change dataobj.text here because it already shows correctly
function TimeToDie:ApplySettings()
   local profile = self.db.profile
	frame:SetFrameStrata(profile.strata)
	if profile.locked then
		bg:Hide()
		frame:EnableMouse(false)
		if text:GetText() == 'TimeToDie' then
			text:SetText()
		end
	else
		bg:Show()
		frame:EnableMouse(true)
		if not text:GetText() then
			text:SetText('TimeToDie')
		end
	end
	text:SetFont(SML and SML:Fetch('font', profile.font) or defaultFont, profile.size, profile.outline)
	text:SetTextColor(profile.r,profile.g,profile.b)
	timer, health0, time0, mhealth, mtime = nil
end

TimeToDie.dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('TimeToDie', {
   type = 'data source',
   text = 'TimeToDie',
   label = 'TimeToDie',
   unit = 'time',
   icon = 'Interface\\Icons\\Spell_Holy_BorrowedTime',
})
